@@ -12,7 +12,7 @@ use DBIx::Log4perl::Constants qw (:masks $LogMask);
 use DBIx::Log4perl::db;
 use DBIx::Log4perl::st;
 
-our $VERSION = '0.12';
+our $VERSION = '0.13';
 require Exporter;
 our @ISA = qw(Exporter DBI);		# look in DBI for anything we don't do
 
@@ -42,16 +42,18 @@ BEGIN {
 my $_glogger;
 
 sub _dbix_l4p_debug {
-    my ($self, $level, $thing, @args) = @_;
+    my ($self, $h, $level, $thing, @args) = @_;
 
-    my $h = $self->{private_DBIx_Log4perl};
+    $h = $self->{private_DBIx_Log4perl} if !defined($h);
 
     return unless $h->{logger}->is_debug();
 
-    $Data::Dumper::Indent = 0;
+    local $Data::Dumper::Indent = 0;
+    local $Data::Dumper::Quotekeys = 0;
+
     local $Log::Log4perl::caller_depth = $Log::Log4perl::caller_depth + $level
         if $level;
-    
+
     if (scalar(@args) > 1) {
 	$h->{logger}->debug(
 	    sub {Data::Dumper->Dump([\@args], [$thing])})
@@ -81,9 +83,9 @@ sub _dbix_l4p_info {
 
     local $Log::Log4perl::caller_depth = $Log::Log4perl::caller_depth + $level
         if $level;
-    
+
     $h->{logger}->info($thing);
- 
+
     return;
 }
 sub _dbix_l4p_warning {
@@ -93,7 +95,8 @@ sub _dbix_l4p_warning {
 
     return unless $h->{logger}->is_warn();
 
-    $Data::Dumper::Indent = 0;
+    local $Data::Dumper::Indent = 0;
+    local $Data::Dumper::Quotekeys = 0;
 
     local $Log::Log4perl::caller_depth = $Log::Log4perl::caller_depth + $level
         if $level;
@@ -117,7 +120,8 @@ sub _dbix_l4p_error {
 
     return unless $h->{logger}->is_error();
 
-    $Data::Dumper::Indent = 0;
+    local $Data::Dumper::Indent = 0;
+    local $Data::Dumper::Quotekeys = 0;
 
     local $Log::Log4perl::caller_depth = $Log::Log4perl::caller_depth + $level
         if $level;
@@ -361,7 +365,7 @@ sub connect {
     }
     $h{logger} = Log::Log4perl->get_logger() if (!exists($h{logger}));
     $_glogger = $h{logger};
-    
+
     my $dbh = $drh->SUPER::connect($dsn, $user, $pass, $attr);
     return $dbh if (!$dbh);
 
@@ -369,8 +373,8 @@ sub connect {
     $dbh->{private_DBIx_Log4perl} = \%h;
     if ($h{logmask} & DBIX_L4P_LOG_CONNECT) {
 	$h{logger}->debug("connect($h{dbh_no}): " .
-                              (defined($dsn) ? $dsn : '') .
-                                  (defined($user) ? $user : ''));
+                              (defined($dsn) ? $dsn : '') . ', ' .
+			  (defined($user) ? $user : ''));
 	no strict 'refs';
 	my $v = "DBD::" . $dbh->{Driver}->{Name} . "::VERSION";
 	$h{logger}->info("DBI: " . $DBI::VERSION,
@@ -424,7 +428,7 @@ parameters, result-sets, transactions etc to a Log::Log4perl handle.
   or
 
   use DBIx::Log4perl;
-  my $dbh = DBIX::Log4perl->connect('DBI:odbc:mydsn', $user, $pass
+  my $dbh = DBIx::Log4perl->connect('DBI:odbc:mydsn', $user, $pass,
                                     {DBIx_l4p_init => "/etc/mylog.conf",
                                      DBIx_l4p_class => "My::Package");
   $dbh->DBI_METHOD(args);
@@ -436,7 +440,8 @@ activity via a Log::Log4perl handle. Log::Log4perl has many advantages
 for logging but the ones probably most attractive are:
 
 The ability to turn logging on or off or change the logging you see
-without changing your code.
+without changing your code (or even without restarting your programs
+if you use C<init_and_watch>).
 
 Different log levels allowing you to separate warnings, errors and fatals
 to different files.
@@ -449,14 +454,14 @@ DBIx::Log4perl adds the following methods over DBI.
 
   $h->dbxi_l4p_getattr('DBIx_l4p_logmask');
 
-Returns the value for a DBIx::Log4perl attribute (see L</Attributes>).
+Returns the value for a DBIx::Log4perl attribute (see L</ATTRIBUTES>).
 
 =head2 dbix_l4p_setattr
 
  $h->dbix_l4p_setattr('DBIx_l4p_logmask', 1);
 
 Set the value of the specified DBIx::Log4perl attribute
-(see L</Attributes>).
+(see L</ATTRIBUTES>).
 
 =head2 dbix_l4p_logdie
 
@@ -466,7 +471,7 @@ Calls the internal _error_handler method with the message $message
 then dies with Carp::confess.
 
 The internal error handler is inserted into DBI's HandleError if
-DBIX_L4P_LOG_ERRCAPTURE is enabled. It attempts to log as much
+L</DBIX_L4P_LOG_ERRCAPTURE> is enabled. It attempts to log as much
 information about the SQL you were executing, parameters etc.
 
 As an example, you might be checking a $dbh->do which attempts to
@@ -490,7 +495,9 @@ may be ORed together to obtain the logging level you require:
 =item DBIX_L4P_LOG_DEFAULT
 
 By default LogMask is set to DBIX_L4P_LOG_DEFAULT which is currently
-DBIX_L4P_LOG_TXN | DBIC_L4P_LOG_CONNECT | DBIX_L4P_LOG_INPUT | DBIX_L4P_LOG_ERRCAPTURE | DBIX_L4P_LOG_ERRORS | DBIX_L4P_LOG_DBDSPECIFIC.
+DBIX_L4P_LOG_TXN | DBIC_L4P_LOG_CONNECT | DBIX_L4P_LOG_INPUT |
+DBIX_L4P_LOG_ERRCAPTURE | DBIX_L4P_LOG_ERRORS |
+DBIX_L4P_LOG_DBDSPECIFIC.
 
 =item DBIX_L4P_LOG_ALL
 
@@ -532,7 +539,7 @@ by RaiseError. Currently this is only the execute_array method.
 
 =item DBIX_L4P_LOG_WARNINGS
 
-Log at Log4perl warning level any calls to do will return no affected
+Log at Log4perl warning level any calls to do which return no affected
 rows on an insert, update or delete opertion.
 
 =item DBIX_L4P_LOG_ERRCAPTURE
@@ -553,7 +560,7 @@ some or all of the following depending on what is available:
 
 If you install your own error handler in the C<connect> call it will
 be replaced when C<connect> is called in DBI but run from
-DBIx::Log4perl's error handler.
+C<DBIx::Log4perl>'s error handler.
 
 C<DBIx::Log4perl> always returns 0 from the error handler if it is the
 only handler which causes the error to be passed on. If you have
@@ -569,9 +576,9 @@ This logging depends on the DBD you are using:
 =item DBD::Oracle
 
 Use DBD::Oracle's methods for obtaining the buffer containing
-dbms_output.put_line output. Whenever C<$dbh-E<gt>execute> is called
+C<dbms_output.put_line output>. Whenever C<$dbh-E<gt>execute> is called
 DBIx::Log4perl will use C<$dbh-E<gt>func('dbms_output_get')> to obtain
-an array of lines written to the buffer with put_line. These will be
+an array of lines written to the buffer with C<put_line>. These will be
 written to the log (prefixed with "dbms") at level DEBUG for the
 execute method.
 
@@ -581,10 +588,15 @@ succeeded. This will use DBD::Oracle's default buffer size. If you want
 to change the buffer size see DBD::Oracle and change it after the connect
 method has returned.
 
-=back
+As useful as this may seem you are warned against using it as when the
+dbms_output buffer is full it will generate an Oracle exception which
+is probably not what you want. This can happen if the procedure you
+call calls dbms_output.put_line too often and fills the buffer before
+returning to DBI.
 
 =back
 
+=back
 
 =head1 ATTRIBUTES
 
@@ -597,21 +609,21 @@ C<DBIx::Log4perl> supports the following attributes:
 
 =item C<DBIx_l4p_init>
 
-This is the string to pass on to Log::Log4Perl's init method. It is
+This is the string to pass on to Log::Log4Perl's C<init> method. It is
 the name of the Log::Log4perl configuration file to use. e.g.
 
   Log::Log4perl::init('/etc/log4perl.conf');
 
-See Log::Log4perl.
+See L<Log::Log4perl>.
 
 =item C<DBIx_l4p_log>
 
-This is the string to pass on to Log::Log4Perl's get_logger method
+This is the string to pass on to Log::Log4Perl's C<get_logger> method
 e.g.
 
   $logger = Log::Log4perl->get_logger('mysys.dbi');
 
-See Log::Log4perl.
+See L<Log::Log4perl>.
 
 =item C<DBIx_l4p_logger>
 
@@ -623,28 +635,28 @@ will ignore DBIx_l4p_log and DBIx_l4p_init.
 
 Although these attributes are supported the recommended way to use
 DBIx::Log4perl it to use Log::Log4perl in your application and call
-the Log::Log4Perl->init to define your log4perl configuration file.
+the C<Log::Log4Perl->init> to define your log4perl configuration file.
 DBIx::Log4perl will then call
-Log::Log4perl->get_logger("DBIx::Log4perl") (as was intended by the
+C<Log::Log4perl->get_logger("DBIx::Log4perl")> (as was intended by the
 authors of Log::Log4perl) and all you need is a
 C<log4perl.logger.DBIx.Log4perl> entry in your configuration file.
 
 =head1 Log::Log4perl CONFIGURATION FILE
 
-Please see Log::Log4perl for full details of the configuration file
+Please see L<Log::Log4perl> for full details of the configuration file
 and appenders. DBIx::Log4perl contains a sample configuration file you
 may use to get started. It looks like this:
 
   log4perl.logger = FATAL, LOGFILE
-      
+
   log4perl.appender.LOGFILE=Log::Log4perl::Appender::File
   log4perl.appender.LOGFILE.filename=/tmp/log
   log4perl.appender.LOGFILE.mode=append
   log4perl.appender.LOGFILE.Threshold = ERROR
-      
+
   log4perl.appender.LOGFILE.layout=PatternLayout
   log4perl.appender.LOGFILE.layout.ConversionPattern=[%r] %F %L %c - %m%n
-  
+
   log4perl.logger.DBIx.Log4perl=DEBUG, A1
   log4perl.appender.A1=Log::Log4perl::Appender::File
   log4perl.appender.A1.filename=/tmp/xlog
@@ -655,12 +667,12 @@ This is perhaps the most simple configuration. It says fatal errors go
 to /tmp/log and debug and above go to /tmp/xlog. It also uses the
 SimpleLayout which prefixes each line with the log level. You can
 use:
- 
+
   log4perl.appender.A1.layout=Log::Log4perl::Layout::PatternLayout
   log4perl.appender.A1.layout.ConversionPattern=%d %p> %F{1}:%L %M - %m%n
 
-to make Log4perl prefix the line with a timestamp, module name and
-filename. DBIx::Log4perl sets $Log::Log4perl::caller_depth in each
+to make Log::Log4perl prefix the line with a timestamp, module name and
+filename. DBIx::Log4perl sets C<$Log::Log4perl::caller_depth> in each
 method so when Log4perl outputs the module/file DBIx::Log4perl
 is ignored.
 
@@ -681,7 +693,11 @@ For
 you will get:
 
   DEBUG - prepare(0.1): 'insert into mytest values (?,?)'
-  DEBUG - $execute(0.1) = [1,'one'];
+  DEBUG - $execute(0.1) (insert into mytest values (?,?)) = [1,'one'];
+
+In this latter case the SQL is repeated for convenience but this only
+occurs if C<execute> is called with parameters. If C<execute> is called
+without any arguments the SQL is no repeated in the C<execute>.
 
 The numbers in the () after a method name indicate which connection or
 statement handle the operation was performed on. The first connection
@@ -696,7 +712,7 @@ actually call all the lower methods because the driver implements
 selectrow_* methods in C. For these cases, DBIx::Log4perl will only be
 able to log the selectrow_* method, the SQL, any parameters and any
 returned result-set and you will not necessarily see a prepare,
-execute and fetch in the log. e.g.:
+execute and fetch in the log. e.g.,
 
   $dbh->selectrow_array('select b from mytest where a = ?',undef,1);
 
@@ -706,7 +722,7 @@ results in:
 
 with no evidence prepare/execute/fetch was called.
 
-If DBIX_L4P_LOG_ERRCAPTURE is set all possible information about an
+If C<DBIX_L4P_LOG_ERRCAPTURE> is set all possible information about an
 error is written to the log by the error handler. In addition a few
 method calls will attempt to write a separate log entry containing
 information which may not be available in the error handler e.g.
@@ -791,9 +807,9 @@ db connection
 
 o the Kids and ActiveKids value for this db - (see DBI docs)
 
-o the error message text in DBI::errstr
+o the error message text in C<DBI::errstr>
 
-o any sql parameters passed to DBI (see DBI docs for ParamValues)
+o any sql parameters passed to DBI (see L<DBI> for ParamValues)
 
 o a trace of where the problem occurred In this case the final problem
   was in db.pm but as this is DBIx::Log4perl's do method, the real
@@ -802,7 +818,7 @@ o a trace of where the problem occurred In this case the final problem
 
 =head2 Use of Data::Dumper
 
-DBIX::log4perl makes extensive use of Data::Dumper to output arguments
+DBIx::log4perl makes extensive use of Data::Dumper to output arguments
 passed to DBI methods. In some cases it combines the method called
 with the data it is logging e.g.
 
@@ -814,7 +830,7 @@ called like this:
 
   Data::Dumper->dump( [ \@execute_args ], [ 'execute'] )
 
-so Data::Dumper believe it is dumping $execute. DBIx::Log4perl uses
+so Data::Dumper believes it is dumping $execute. DBIx::Log4perl uses
 this method extensively to log the method and arguments - just ignore
 the leading '$' in the log.
 
@@ -878,6 +894,75 @@ I had to add the following to dbdimp.c to make it work:
 
 I believe this code is now added in DBD::mysql 3.0003_1.
 
+=head1 Contributing
+
+There are six main ways you may help with the development and
+maintenance of this module:
+
+=over
+
+=item Submitting patches
+
+Please get the latest version from CPAN and submit any patches against
+that.
+
+=item Reporting installs
+
+Install CPAN::Reporter and report you installations. This is easy to
+do - see L</CPAN Testers Reporting>.
+
+=item Report bugs
+
+If you find what you believe is a bug then enter it into the
+L<http://rt.cpan.org/Dist/Display.html?Name=DBIx::Log4perl>
+system. Where possible include code which reproduces the problem
+including any schema required and the versions of software you are
+using.
+
+If you are unsure whether you have found a bug report it anyway.
+
+=item pod comments and corrections
+
+If you find inaccuracies in the DBIx::Log4perl pod or have a comment
+which you think should be added then go to L<http://annocpan.org> and
+submit them there. I get an email for every comment added and will
+review each one and apply any changes to the documentation.
+
+=item Review DBIx::Log4perl
+
+Add your review of DBIx::Log4perl on L<http://cpanratings.perl.org>.
+
+=item submit test cases
+
+The test suite for DBIx::Log4perl is pitifully small. Any test cases
+would be gratefully received.
+
+=back
+
+=head1 CPAN Testers Reporting
+
+Please, please, please (is that enough), consider installing
+CPAN::Reporter so that when you install perl modules a report of the
+installation success or failure can be sent to cpan testers. In this
+way module authors 1) get feedback on the fact that a module is being
+installed 2) get to know if there are any installation problems. Also
+other people like you may look at the test reports to see how
+successful they are before choosing the version of a module to
+install.
+
+CPAN::Reporter is easy to install and configure like this:
+
+  perl -MCPAN -e shell
+  cpan> install CPAN::Reporter
+  cpan> reload cpan
+  cpan> o conf init test_report
+
+Simply answer the questions to configure CPAN::Reporter.
+
+You can find the CPAN testers wiki at L<http://wiki.cpantesters.org/>
+and the installation guide for CPAN::Reporter at
+L<http://wiki.cpantesters.org/wiki/CPANInstall>.
+
 =head1 REQUIREMENTS
 
 You will need at least Log::Log4perl 1.04 and DBI 1.50.
@@ -894,9 +979,9 @@ The same applies to logdie. See the Log4perl mailing list for details.
 
 =head1 SEE ALSO
 
-DBI
+L<DBI>
 
-Log::Log4perl
+L<Log::Log4perl>
 
 =head1 AUTHOR
 
@@ -904,11 +989,10 @@ M. J. Evans, E<lt>mjevans@cpan.orgE<gt>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (C) 2006 by M. J. Evans
+Copyright (C) 2006, 2007, 2008 by M. J. Evans
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself, either Perl version 5.8.7 or,
 at your option, any later version of Perl 5 you may have available.
 
-
 =cut
